@@ -4,15 +4,23 @@
 
 /* import-globals-from pageInfo.js */
 
-const { SitePermissions } = ChromeUtils.import(
-  "resource:///modules/SitePermissions.jsm"
+const { SitePermissions } = ChromeUtils.importESModule(
+  "resource:///modules/SitePermissions.sys.mjs"
 );
 
 var gPermPrincipal;
 
+// List of ids of permissions to hide.
+const EXCLUDE_PERMS = ["open-protocol-handler"];
+
 // Array of permissionIDs sorted alphabetically by label.
-var gPermissions = SitePermissions.listPermissions()
-  .filter(p => SitePermissions.getPermissionLabel(p) != null)
+let gPermissions = SitePermissions.listPermissions()
+  .filter(permissionID => {
+    if (!SitePermissions.getPermissionLabel(permissionID)) {
+      return false;
+    }
+    return !EXCLUDE_PERMS.includes(permissionID);
+  })
   .sort((a, b) => {
     let firstLabel = SitePermissions.getPermissionLabel(a);
     let secondLabel = SitePermissions.getPermissionLabel(b);
@@ -33,6 +41,10 @@ var permissionObserver = {
   },
 };
 
+function getExcludedPermissions() {
+  return EXCLUDE_PERMS;
+}
+
 function onLoadPermission(uri, principal) {
   var permTab = document.getElementById("permTab");
   if (SitePermissions.isSupportedPrincipal(principal)) {
@@ -169,6 +181,7 @@ function createRow(aPartId) {
   let checkbox = document.createXULElement("checkbox");
   checkbox.setAttribute("id", aPartId + "Def");
   checkbox.setAttribute("oncommand", "onCheckboxClick('" + aPartId + "');");
+  checkbox.setAttribute("native", true);
   document.l10n.setAttributes(checkbox, "permissions-use-default");
   controls.appendChild(checkbox);
 
