@@ -4,19 +4,31 @@
  * Test searching for the selected text using the context menu
  */
 
+const { AddonTestUtils } = ChromeUtils.importESModule(
+  "resource://testing-common/AddonTestUtils.sys.mjs"
+);
+
+AddonTestUtils.initMochitest(this);
+
 const ENGINE_NAME = "mozSearch";
-const ENGINE_ID = "mozsearch-engine@search.mozilla.org";
 const PRIVATE_ENGINE_NAME = "mozPrivateSearch";
-const PRIVATE_ENGINE_ID = "mozsearch-engine-private@search.mozilla.org";
+const ENGINE_DATA = new Map([
+  [
+    ENGINE_NAME,
+    "https://example.com/browser/browser/components/search/test/browser/mozsearch.sjs",
+  ],
+  [PRIVATE_ENGINE_NAME, "https://example.com:443/browser/"],
+]);
 
 let engine;
 let privateEngine;
+let extensions = [];
+let oldDefaultEngine;
+let oldDefaultPrivateEngine;
 
-add_task(async function setup() {
+add_setup(async function () {
   await SpecialPowers.pushPrefEnv({
     set: [
-      // We want select events to be fired.
-      ["dom.select_events.enabled", true],
       ["browser.search.separatePrivateDefault", true],
       ["browser.search.separatePrivateDefault.ui.enabled", true],
     ],
@@ -24,42 +36,44 @@ add_task(async function setup() {
 
   await Services.search.init();
 
-  // replace the path we load search engines from with
-  // the path to our test data.
-  let searchExtensions = getChromeDir(getResolvedURI(gTestPath));
-  searchExtensions.append("mozsearch");
-  let resProt = Services.io
-    .getProtocolHandler("resource")
-    .QueryInterface(Ci.nsIResProtocolHandler);
-  let originalSubstitution = resProt.getSubstitution("search-extensions");
-  resProt.setSubstitution(
-    "search-extensions",
-    Services.io.newURI("file://" + searchExtensions.path)
-  );
+  for (let [name, search_url] of ENGINE_DATA) {
+    let extension = ExtensionTestUtils.loadExtension({
+      manifest: {
+        chrome_settings_overrides: {
+          search_provider: {
+            name,
+            search_url,
+            params: [
+              {
+                name: "test",
+                value: "{searchTerms}",
+              },
+            ],
+          },
+        },
+      },
+    });
 
-  await Services.search.ensureBuiltinExtension(ENGINE_ID);
-  await Services.search.ensureBuiltinExtension(PRIVATE_ENGINE_ID);
+    await extension.startup();
+    await AddonTestUtils.waitForSearchProviderStartup(extension);
+    extensions.push(extension);
+  }
 
   engine = await Services.search.getEngineByName(ENGINE_NAME);
   Assert.ok(engine, "Got a search engine");
-  let defaultEngine = await Services.search.getDefault();
-  await Services.search.setDefault(engine);
+  oldDefaultEngine = await Services.search.getDefault();
+  await Services.search.setDefault(
+    engine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 
   privateEngine = await Services.search.getEngineByName(PRIVATE_ENGINE_NAME);
   Assert.ok(privateEngine, "Got a search engine");
-  let defaultPrivateEngine = await Services.search.getDefaultPrivate();
-  await Services.search.setDefaultPrivate(privateEngine);
-
-  registerCleanupFunction(async () => {
-    await Services.search.setDefault(defaultEngine);
-    await Services.search.setDefaultPrivate(defaultPrivateEngine);
-    await Services.search.removeEngine(engine);
-    await Services.search.removeEngine(privateEngine);
-
-    resProt.setSubstitution("search-extensions", originalSubstitution);
-    // Finial re-init to make sure we drop the added built-in engines.
-    await Services.search.reInit();
-  });
+  oldDefaultPrivateEngine = await Services.search.getDefaultPrivate();
+  await Services.search.setDefaultPrivate(
+    privateEngine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 });
 
 async function checkContextMenu(
@@ -76,11 +90,11 @@ async function checkContextMenu(
     "https://example.com/browser/browser/components/search/test/browser/test_search.html"
   );
 
-  await SpecialPowers.spawn(tab.linkedBrowser, [""], async function() {
+  await SpecialPowers.spawn(tab.linkedBrowser, [""], async function () {
     return new Promise(resolve => {
       content.document.addEventListener(
         "selectionchange",
-        function() {
+        function () {
           resolve();
         },
         { once: true }
@@ -118,13 +132,13 @@ async function checkContextMenu(
 
   let loaded = BrowserTestUtils.waitForNewTab(
     win.gBrowser,
-    expectedBaseUrl,
+    expectedBaseUrl + "?test=test%2520search",
     true
   );
-  searchItem.click();
+  contextMenu.activateItem(searchItem);
   let searchTab = await loaded;
   let browser = win.gBrowser.selectedBrowser;
-  await SpecialPowers.spawn(browser, [], async function() {
+  await SpecialPowers.spawn(browser, [], async function () {
     Assert.ok(
       !/error/.test(content.document.body.innerHTML),
       "Ensure there were no errors loading the search page"
@@ -182,7 +196,10 @@ add_task(async function test_privateWindow() {
 add_task(async function test_normalWindow_sameDefaults() {
   // Set the private default engine to be the same as the current default engine
   // in 'normal' mode.
-  await Services.search.setDefaultPrivate(await Services.search.getDefault());
+  await Services.search.setDefaultPrivate(
+    await Services.search.getDefault(),
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
 
   await checkContextMenu(
     window,
@@ -211,3 +228,22 @@ add_task(async function test_privateWindow_no_separate_engine() {
     "https://example.com/browser/browser/components/search/test/browser/mozsearch.sjs"
   );
 });
+
+// We can't do the unload within registerCleanupFunction as that's too late for
+// the test to be happy. Do it into a cleanup "test" here instead.
+add_task(async function cleanup() {
+  await Services.search.setDefault(
+    oldDefaultEngine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
+  await Services.search.setDefaultPrivate(
+    oldDefaultPrivateEngine,
+    Ci.nsISearchService.CHANGE_REASON_UNKNOWN
+  );
+  await Services.search.removeEngine(engine);
+  await Services.search.removeEngine(privateEngine);
+
+  for (let extension of extensions) {
+    await extension.unload();
+  }
+});
