@@ -1,5 +1,7 @@
 "use strict";
 
+requestLongerTimeout(2);
+
 const MAX_CONCURRENT_TABS = "browser.engagement.max_concurrent_tab_count";
 const TAB_EVENT_COUNT = "browser.engagement.tab_open_event_count";
 const MAX_CONCURRENT_WINDOWS = "browser.engagement.max_concurrent_window_count";
@@ -22,17 +24,22 @@ ChromeUtils.defineModuleGetter(
   "resource:///modules/BrowserUsageTelemetry.jsm"
 );
 
-const { SessionStore } = ChromeUtils.import(
-  "resource:///modules/sessionstore/SessionStore.jsm"
+const { ObjectUtils } = ChromeUtils.import(
+  "resource://gre/modules/ObjectUtils.jsm"
 );
 
+BrowserUsageTelemetry._onTabsOpenedTask._timeoutMs = 0;
+registerCleanupFunction(() => {
+  BrowserUsageTelemetry._onTabsOpenedTask._timeoutMs = undefined;
+});
+
 // Reset internal URI counter in case URIs were opened by other tests.
 Services.obs.notifyObservers(null, TELEMETRY_SUBSESSION_TOPIC);
 
 /**
  * Get a snapshot of the scalars and check them against the provided values.
  */
-let checkScalars = countsObject => {
+let checkScalars = (countsObject, skipGleanCheck = false) => {
   const scalars = TelemetryTestUtils.getProcessScalars("parent");
 
   // Check the expected values. Scalars that are never set must not be reported.
@@ -90,11 +97,27 @@ let checkScalars = countsObject => {
     countsObject.totalURIsNormalAndPrivateMode,
     "The total URI count for both normal and private mode must match the expected value."
   );
+  if (!skipGleanCheck) {
+    if (countsObject.totalURIsNormalAndPrivateMode == 0) {
+      Assert.equal(
+        Glean.browserEngagement.uriCount.testGetValue(),
+        undefined,
+        "Total URI count reported in Glean must be unset."
+      );
+    } else {
+      Assert.equal(
+        countsObject.totalURIsNormalAndPrivateMode,
+        Glean.browserEngagement.uriCount.testGetValue(),
+        "The total URI count reported in Glean must be as expected."
+      );
+    }
+  }
 };
 
 add_task(async function test_tabsAndWindows() {
   // Let's reset the counts.
   Services.telemetry.clearScalars();
+  Services.fog.testResetFOG();
 
   let openedTabs = [];
   let expectedTabOpenCount = 0;
@@ -164,6 +187,7 @@ add_task(async function test_tabsAndWindows() {
 
   // Add a new window and then some tabs in it. An empty new windows counts as a tab.
   let win = await BrowserTestUtils.openNewBrowserWindow();
+  await BrowserTestUtils.firstBrowserLoaded(win);
   openedTabs.push(
     await BrowserTestUtils.openNewForegroundTab(win.gBrowser, "about:blank")
   );
@@ -221,6 +245,7 @@ add_task(async function test_subsessionSplit() {
 
   // Add a new window (that will have 4 tabs).
   let win = await BrowserTestUtils.openNewBrowserWindow();
+  await BrowserTestUtils.firstBrowserLoaded(win);
   let openedTabs = [];
   openedTabs.push(
     await BrowserTestUtils.openNewForegroundTab(win.gBrowser, "about:blank")
@@ -266,18 +291,21 @@ add_task(async function test_subsessionSplit() {
   // must not be reported.
   expectedTotalURIs = 0;
 
-  checkScalars({
-    maxTabs: 4,
-    tabOpenCount: 0,
-    maxWindows: 2,
-    windowsOpenCount: 0,
-    totalURIs: expectedTotalURIs,
-    domainCount: 0,
-    totalUnfilteredURIs: 0,
-    maxTabsPinned: 0,
-    tabPinnedCount: 0,
-    totalURIsNormalAndPrivateMode: expectedTotalURIs,
-  });
+  checkScalars(
+    {
+      maxTabs: 4,
+      tabOpenCount: 0,
+      maxWindows: 2,
+      windowsOpenCount: 0,
+      totalURIs: expectedTotalURIs,
+      domainCount: 0,
+      totalUnfilteredURIs: 0,
+      maxTabsPinned: 0,
+      tabPinnedCount: 0,
+      totalURIsNormalAndPrivateMode: expectedTotalURIs,
+    },
+    true
+  );
 
   // Remove all the extra windows and tabs.
   for (let tab of openedTabs) {
@@ -319,8 +347,12 @@ add_task(async function test_tabsHistogram() {
   );
   openedTabs.push(tab);
   BrowserUsageTelemetry._lastRecordTabCount = 0;
-  BrowserTestUtils.loadURI(tab.linkedBrowser, "http://example.com/");
-  await BrowserTestUtils.browserLoaded(tab.linkedBrowser);
+  BrowserTestUtils.loadURIString(tab.linkedBrowser, "http://example.com/");
+  await BrowserTestUtils.browserLoaded(
+    tab.linkedBrowser,
+    false,
+    "http://example.com/"
+  );
   checkTabCountHistogram(
     tabCountHist.snapshot(),
     { 1: 0, 2: 1, 3: 2, 4: 0 },
@@ -341,6 +373,7 @@ add_task(async function test_tabsHistogram() {
   // Add a new window and then some tabs in it. A new window starts with one tab.
   BrowserUsageTelemetry._lastRecordTabCount = 0;
   let win = await BrowserTestUtils.openNewBrowserWindow();
+  await BrowserTestUtils.firstBrowserLoaded(win);
   checkTabCountHistogram(
     tabCountHist.snapshot(),
     { 1: 0, 2: 1, 3: 2, 4: 1, 5: 1, 6: 0 },
@@ -395,8 +428,12 @@ add_task(async function test_tabsHistogram() {
     Date.now() - MINIMUM_TAB_COUNT_INTERVAL_MS / 2;
   {
     let oldLastRecordTabCount = BrowserUsageTelemetry._lastRecordTabCount;
-    BrowserTestUtils.loadURI(tab.linkedBrowser, "http://example.com/");
-    await BrowserTestUtils.browserLoaded(tab.linkedBrowser);
+    BrowserTestUtils.loadURIString(tab.linkedBrowser, "http://example.com/");
+    await BrowserTestUtils.browserLoaded(
+      tab.linkedBrowser,
+      false,
+      "http://example.com/"
+    );
     checkTabCountHistogram(
       tabCountHist.snapshot(),
       { 1: 0, 2: 1, 3: 2, 4: 1, 5: 1, 7: 1, 8: 0 },
@@ -413,8 +450,12 @@ add_task(async function test_tabsHistogram() {
     Date.now() - (MINIMUM_TAB_COUNT_INTERVAL_MS + 1000);
   {
     let oldLastRecordTabCount = BrowserUsageTelemetry._lastRecordTabCount;
-    BrowserTestUtils.loadURI(tab.linkedBrowser, "http://example.com/");
-    await BrowserTestUtils.browserLoaded(tab.linkedBrowser);
+    BrowserTestUtils.loadURIString(tab.linkedBrowser, "http://example.com/");
+    await BrowserTestUtils.browserLoaded(
+      tab.linkedBrowser,
+      false,
+      "http://example.com/"
+    );
     checkTabCountHistogram(
       tabCountHist.snapshot(),
       { 1: 0, 2: 1, 3: 2, 4: 1, 5: 1, 7: 2, 8: 0 },
@@ -451,9 +492,8 @@ add_task(async function test_loadedTabsHistogram() {
 
   resetTimestamps();
   const tabCount = TelemetryTestUtils.getAndClearHistogram("TAB_COUNT");
-  const loadedTabCount = TelemetryTestUtils.getAndClearHistogram(
-    "LOADED_TAB_COUNT"
-  );
+  const loadedTabCount =
+    TelemetryTestUtils.getAndClearHistogram("LOADED_TAB_COUNT");
 
   checkTabCountHistogram(tabCount.snapshot(), {}, "TAB_COUNT - initial count");
   checkTabCountHistogram(
@@ -468,16 +508,8 @@ add_task(async function test_loadedTabsHistogram() {
   ];
 
   // There are two tabs open: the mochi.test tab and the foreground tab.
-  checkTabCountHistogram(
-    tabCount.snapshot(),
-    { 1: 0, 2: 1, 3: 0 },
-    "TAB_COUNT - new tab"
-  );
-  checkTabCountHistogram(
-    loadedTabCount.snapshot(),
-    { 1: 0, 2: 1, 3: 0 },
-    "TAB_COUNT - new tab"
-  );
+  const snapshot = loadedTabCount.snapshot();
+  checkTabCountHistogram(snapshot, { 1: 0, 2: 1, 3: 0 }, "TAB_COUNT - new tab");
 
   // Open a pending tab, as if by session restore.
   resetTimestamps();
@@ -486,6 +518,10 @@ add_task(async function test_loadedTabsHistogram() {
   });
   tabs.push(lazyTab);
 
+  await BrowserTestUtils.waitForCondition(
+    () => !ObjectUtils.deepEqual(snapshot, tabCount.snapshot())
+  );
+
   checkTabCountHistogram(
     tabCount.snapshot(),
     { 1: 0, 2: 1, 3: 1, 4: 0 },
@@ -519,7 +555,10 @@ add_task(async function test_loadedTabsHistogram() {
   resetTimestamps();
 
   await Promise.all([
-    BrowserTestUtils.loadURI(lazyTab.linkedBrowser, "http://example.com/"),
+    BrowserTestUtils.loadURIString(
+      lazyTab.linkedBrowser,
+      "http://example.com/"
+    ),
     BrowserTestUtils.browserLoaded(
       lazyTab.linkedBrowser,
       false,
@@ -541,6 +580,7 @@ add_task(async function test_loadedTabsHistogram() {
 
   resetTimestamps();
   let win = await BrowserTestUtils.openNewBrowserWindow();
+  await BrowserTestUtils.firstBrowserLoaded(win);
 
   // The new window will have a new tab.
   checkTabCountHistogram(
@@ -580,8 +620,8 @@ add_task(async function test_restored_max_pinned_count() {
   // Following pinned tab testing example from
   // https://searchfox.org/mozilla-central/rev/1843375acbbca68127713e402be222350ac99301/browser/components/sessionstore/test/browser_pinned_tabs.js
   Services.telemetry.clearScalars();
-  const { E10SUtils } = ChromeUtils.import(
-    "resource://gre/modules/E10SUtils.jsm"
+  const { E10SUtils } = ChromeUtils.importESModule(
+    "resource://gre/modules/E10SUtils.sys.mjs"
   );
   const BACKUP_STATE = SessionStore.getBrowserState();
   const triggeringPrincipal_base64 = E10SUtils.SERIALIZED_SYSTEMPRINCIPAL;
