@@ -40,13 +40,13 @@ import android.net.Network;
 import android.net.NetworkInfo;
 import android.os.Build;
 import android.os.Bundle;
+import android.os.Debug;
 import android.os.LocaleList;
 import android.os.Looper;
 import android.os.PowerManager;
 import android.os.Vibrator;
 import android.provider.Settings;
 import android.text.TextUtils;
-import android.text.format.DateFormat;
 import android.util.Log;
 import android.view.ContextThemeWrapper;
 import android.view.Display;
@@ -61,6 +61,7 @@ import java.nio.ByteBuffer;
 import java.util.List;
 import java.util.Locale;
 import java.util.StringTokenizer;
+import org.jetbrains.annotations.NotNull;
 import org.mozilla.gecko.annotation.RobocopTarget;
 import org.mozilla.gecko.annotation.WrapForJNI;
 import org.mozilla.gecko.util.HardwareCodecCapabilityUtils;
@@ -140,7 +141,7 @@ public class GeckoAppShell {
       }
       return false;
     }
-  };
+  }
 
   private static String sAppNotes;
   private static CrashHandler sCrashHandler;
@@ -179,6 +180,8 @@ public class GeckoAppShell {
   }
 
   private static volatile boolean locationHighAccuracyEnabled;
+  private static volatile boolean locationListeningRequested = false;
+  private static volatile boolean locationPaused = false;
 
   // See also HardwareUtils.LOW_MEMORY_THRESHOLD_MB.
   private static final int HIGH_MEMORY_DEVICE_THRESHOLD_MB = 768;
@@ -187,6 +190,7 @@ public class GeckoAppShell {
   private static Float sDensity;
   private static int sScreenDepth;
   private static boolean sUseMaxScreenDepth;
+  private static Float sScreenRefreshRate;
 
   /* Is the value in sVibrationEndTime valid? */
   private static boolean sVibrationMaybePlaying;
@@ -281,9 +285,23 @@ public class GeckoAppShell {
     return (location.hasAccuracy() && radius > 0) ? radius : 1001;
   }
 
+  private static Location determineReliableLocation(
+      @NotNull final Location locA, @NotNull final Location locB) {
+    // The 6 seconds were chosen arbitrarily
+    final long closeTime = 6000000000L;
+    final boolean isNearSameTime =
+        Math.abs((locA.getElapsedRealtimeNanos() - locB.getElapsedRealtimeNanos())) <= closeTime;
+    final boolean isAMoreAccurate = getLocationAccuracy(locA) < getLocationAccuracy(locB);
+    final boolean isAMoreRecent = locA.getElapsedRealtimeNanos() > locB.getElapsedRealtimeNanos();
+    if (isNearSameTime) {
+      return isAMoreAccurate ? locA : locB;
+    }
+    return isAMoreRecent ? locA : locB;
+  }
+
   // Permissions are explicitly checked when requesting content permission.
   @SuppressLint("MissingPermission")
-  private static Location getLastKnownLocation(final LocationManager lm) {
+  private static @Nullable Location getLastKnownLocation(final LocationManager lm) {
     Location lastKnownLocation = null;
     final List<String> providers = lm.getAllProviders();
 
@@ -297,28 +315,37 @@ public class GeckoAppShell {
         lastKnownLocation = location;
         continue;
       }
-
-      final long timeDiff = location.getTime() - lastKnownLocation.getTime();
-      if (timeDiff > 0
-          || (timeDiff == 0
-              && getLocationAccuracy(location) < getLocationAccuracy(lastKnownLocation))) {
-        lastKnownLocation = location;
-      }
+      lastKnownLocation = determineReliableLocation(lastKnownLocation, location);
     }
-
     return lastKnownLocation;
   }
 
+  // Toggles the location listeners on/off, which will then provide/stop location information
   @WrapForJNI(calledFrom = "gecko")
+  private static synchronized boolean enableLocationUpdates(final boolean enable) {
+    locationListeningRequested = enable;
+    final boolean canListen = updateLocationListeners();
+    if (!canListen && locationListeningRequested) {
+      // Didn't successfully start listener when requested
+      locationListeningRequested = false;
+    }
+    return canListen;
+  }
+
   // Permissions are explicitly checked when requesting content permission.
   @SuppressLint("MissingPermission")
-  private static synchronized boolean enableLocation(final boolean enable) {
+  private static synchronized boolean updateLocationListeners() {
+    final boolean shouldListen = locationListeningRequested && !locationPaused;
     final LocationManager lm = getLocationManager(getApplicationContext());
     if (lm == null) {
       return false;
     }
 
-    if (!enable) {
+    if (!shouldListen) {
+      // Could not complete request, because paused
+      if (locationListeningRequested) {
+        return false;
+      }
       lm.removeUpdates(sAndroidListeners);
       return true;
     }
@@ -339,12 +366,8 @@ public class GeckoAppShell {
     criteria.setAltitudeRequired(false);
     if (locationHighAccuracyEnabled) {
       criteria.setAccuracy(Criteria.ACCURACY_FINE);
-      criteria.setCostAllowed(true);
-      criteria.setPowerRequirement(Criteria.POWER_HIGH);
     } else {
       criteria.setAccuracy(Criteria.ACCURACY_COARSE);
-      criteria.setCostAllowed(false);
-      criteria.setPowerRequirement(Criteria.POWER_LOW);
     }
 
     final String provider = lm.getBestProvider(criteria, true);
@@ -357,6 +380,16 @@ public class GeckoAppShell {
     return true;
   }
 
+  public static void pauseLocation() {
+    locationPaused = true;
+    updateLocationListeners();
+  }
+
+  public static void resumeLocation() {
+    locationPaused = false;
+    updateLocationListeners();
+  }
+
   private static LocationManager getLocationManager(final Context context) {
     try {
       return (LocationManager) context.getSystemService(Context.LOCATION_SERVICE);
@@ -387,8 +420,7 @@ public class GeckoAppShell {
       float accuracy,
       float altitudeAccuracy,
       float heading,
-      float speed,
-      long time);
+      float speed);
 
   private static class AndroidListeners implements SensorEventListener, LocationListener {
     @Override
@@ -485,8 +517,7 @@ public class GeckoAppShell {
           accuracy,
           altitudeAccuracy,
           heading,
-          speed,
-          location.getTime());
+          speed);
     }
 
     @Override
@@ -505,10 +536,13 @@ public class GeckoAppShell {
 
   /** Wake-lock for the CPU. */
   static final String WAKE_LOCK_CPU = "cpu";
+
   /** Wake-lock for the screen. */
   static final String WAKE_LOCK_SCREEN = "screen";
+
   /** Wake-lock for the audio-playing, eqaul to LOCK_CPU. */
   static final String WAKE_LOCK_AUDIO_PLAYING = "audio-playing";
+
   /** Wake-lock for the video-playing, eqaul to LOCK_SCREEN.. */
   static final String WAKE_LOCK_VIDEO_PLAYING = "video-playing";
 
@@ -516,8 +550,10 @@ public class GeckoAppShell {
 
   /** No one holds the wake-lock. */
   static final int WAKE_LOCK_STATE_UNLOCKED = 0;
+
   /** The wake-lock is held by a foreground window. */
   static final int WAKE_LOCK_STATE_LOCKED_FOREGROUND = 1;
+
   /** The wake-lock is held by a background window. */
   static final int WAKE_LOCK_STATE_LOCKED_BACKGROUND = 2;
 
@@ -853,6 +889,24 @@ public class GeckoAppShell {
     return sScreenDepth;
   }
 
+  @WrapForJNI(calledFrom = "gecko")
+  public static synchronized float getScreenRefreshRate() {
+    if (sScreenRefreshRate != null) {
+      return sScreenRefreshRate;
+    }
+
+    final WindowManager wm =
+        (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
+    final float refreshRate = wm.getDefaultDisplay().getRefreshRate();
+    // Android 11+ supports multiple refresh rate. So we have to get refresh rate per call.
+    // https://source.android.com/docs/core/graphics/multiple-refresh-rate
+    if (Build.VERSION.SDK_INT < 30) {
+      // Until Android 10, refresh rate is fixed, so we can cache it.
+      sScreenRefreshRate = Float.valueOf(refreshRate);
+    }
+    return refreshRate;
+  }
+
   @WrapForJNI(calledFrom = "gecko")
   private static void performHapticFeedback(final boolean aIsLongPress) {
     // Don't perform haptic feedback if a vibration is currently playing,
@@ -998,7 +1052,7 @@ public class GeckoAppShell {
     }
   }
 
-  @WrapForJNI(calledFrom = "gecko")
+  @WrapForJNI(calledFrom = "gecko", exceptionMode = "nsresult")
   private static String getDNSDomains() {
     if (Build.VERSION.SDK_INT < 23) {
       return "";
@@ -1018,9 +1072,10 @@ public class GeckoAppShell {
     return lp.getDomains();
   }
 
+  @SuppressLint("ResourceType")
   @WrapForJNI(calledFrom = "gecko")
   private static int[] getSystemColors() {
-    // attrsAppearance[] must correspond to AndroidSystemColors structure in android/AndroidBridge.h
+    // attrsAppearance[] must correspond to AndroidSystemColors structure in android/nsLookAndFeel.h
     final int[] attrsAppearance = {
       android.R.attr.textColorPrimary,
       android.R.attr.textColorPrimaryInverse,
@@ -1041,8 +1096,7 @@ public class GeckoAppShell {
     final ContextThemeWrapper contextThemeWrapper =
         new ContextThemeWrapper(getApplicationContext(), android.R.style.TextAppearance);
 
-    final TypedArray appearance =
-        contextThemeWrapper.getTheme().obtainStyledAttributes(attrsAppearance);
+    final TypedArray appearance = contextThemeWrapper.obtainStyledAttributes(attrsAppearance);
 
     if (appearance != null) {
       for (int i = 0; i < appearance.getIndexCount(); i++) {
@@ -1149,6 +1203,7 @@ public class GeckoAppShell {
   }
 
   private static Context sApplicationContext;
+  private static Boolean sIs24HourFormat = true;
 
   @WrapForJNI
   public static Context getApplicationContext() {
@@ -1181,7 +1236,7 @@ public class GeckoAppShell {
   @WrapForJNI(calledFrom = "gecko")
   @RobocopTarget
   public static boolean isTablet() {
-    return HardwareUtils.isTablet();
+    return HardwareUtils.isTablet(getApplicationContext());
   }
 
   @WrapForJNI(calledFrom = "gecko")
@@ -1210,6 +1265,10 @@ public class GeckoAppShell {
     return GeckoScreenOrientation.getInstance().getScreenOrientation().value;
   }
 
+  /* package */ static int getRotation() {
+    return sScreenCompat.getRotation();
+  }
+
   @WrapForJNI(calledFrom = "gecko")
   private static int getScreenAngle() {
     return GeckoScreenOrientation.getInstance().getAngle();
@@ -1335,6 +1394,8 @@ public class GeckoAppShell {
 
   private interface ScreenCompat {
     Rect getScreenSize();
+
+    int getRotation();
   }
 
   @TargetApi(Build.VERSION_CODES.JELLY_BEAN)
@@ -1345,6 +1406,12 @@ public class GeckoAppShell {
       final Display disp = wm.getDefaultDisplay();
       return new Rect(0, 0, disp.getWidth(), disp.getHeight());
     }
+
+    public int getRotation() {
+      final WindowManager wm =
+          (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
+      return wm.getDefaultDisplay().getRotation();
+    }
   }
 
   @TargetApi(Build.VERSION_CODES.JELLY_BEAN_MR1)
@@ -1357,6 +1424,12 @@ public class GeckoAppShell {
       disp.getRealSize(size);
       return new Rect(0, 0, size.x, size.y);
     }
+
+    public int getRotation() {
+      final WindowManager wm =
+          (WindowManager) getApplicationContext().getSystemService(Context.WINDOW_SERVICE);
+      return wm.getDefaultDisplay().getRotation();
+    }
   }
 
   @TargetApi(Build.VERSION_CODES.S)
@@ -1380,6 +1453,11 @@ public class GeckoAppShell {
       final WindowManager windowManager = getWindowContext().getSystemService(WindowManager.class);
       return windowManager.getCurrentWindowMetrics().getBounds();
     }
+
+    public int getRotation() {
+      final WindowManager windowManager = getWindowContext().getSystemService(WindowManager.class);
+      return windowManager.getDefaultDisplay().getRotation();
+    }
   }
 
   static {
@@ -1454,6 +1532,8 @@ public class GeckoAppShell {
     try {
       if (on) {
         Log.e(LOGTAG, "Setting communication mode ON");
+        // This shouldn't throw, but does throw NullPointerException on a very
+        // small number of devices.
         am.startBluetoothSco();
         am.setBluetoothScoOn(true);
       } else {
@@ -1461,7 +1541,7 @@ public class GeckoAppShell {
         am.stopBluetoothSco();
         am.setBluetoothScoOn(false);
       }
-    } catch (final SecurityException e) {
+    } catch (final SecurityException | NullPointerException e) {
       Log.e(LOGTAG, "could not set communication mode", e);
     }
   }
@@ -1502,10 +1582,13 @@ public class GeckoAppShell {
     return locales;
   }
 
+  public static void setIs24HourFormat(final Boolean is24HourFormat) {
+    sIs24HourFormat = is24HourFormat;
+  }
+
   @WrapForJNI
   public static boolean getIs24HourFormat() {
-    final Context context = getApplicationContext();
-    return DateFormat.is24HourFormat(context);
+    return sIs24HourFormat;
   }
 
   @WrapForJNI
@@ -1516,6 +1599,26 @@ public class GeckoAppShell {
     return id == 0 ? info.nonLocalizedLabel.toString() : context.getString(id);
   }
 
+  @WrapForJNI(calledFrom = "gecko")
+  private static int getMemoryUsage(final String stateName) {
+    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.M) {
+      // No API to get Java heap usages.
+      return -1;
+    }
+
+    final Debug.MemoryInfo memInfo = new Debug.MemoryInfo();
+    Debug.getMemoryInfo(memInfo);
+    final String usage = memInfo.getMemoryStat(stateName);
+    if (usage == null) {
+      return -1;
+    }
+    try {
+      return Integer.parseInt(usage);
+    } catch (final NumberFormatException e) {
+      return -1;
+    }
+  }
+
   @WrapForJNI
   public static native boolean isParentProcess();
 
@@ -1525,4 +1628,14 @@ public class GeckoAppShell {
    */
   @WrapForJNI
   public static native GeckoResult<Boolean> isGpuProcessEnabled();
+
+  @SuppressLint("NewApi")
+  public static boolean isIsolatedProcess() {
+    if (Build.VERSION.SDK_INT < Build.VERSION_CODES.JELLY_BEAN) {
+      return false;
+    }
+    // This method was added in SDK 16 but remained hidden until SDK 28, meaning we are okay to call
+    // this on any SDK level but must suppress the new API lint.
+    return android.os.Process.isIsolated();
+  }
 }
